import { useState, useCallback, useMemo } from 'react';
import { max, scaleTime, scaleLog, extent, line } from 'd3';
import { XAxis } from './XAxis';
import { YAxis } from './YAxis';
import { VoronoiOverlay } from './VoronoiOverlay';

const xValue = (d) => d.date;
const yValue = (d) => d.deathTotal;

const margin = {
  top: 40,
  right: 20,
  bottom: 80,
  left: 80,
};

const epsilon = 1;

export const LineChart = ({ data, width, height }) => {
  const [
    activeCountryName,
    setActiveCountryName,
  ] = useState();

  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  const allData = useMemo(
    () =>
      data.reduce(
        (accumulator, currentValue) =>
          accumulator.concat(currentValue),
        []
      ),
    [data]
  );

  const xScale = useMemo(
    () =>
      scaleTime()
        .domain(extent(allData, xValue))
        .range([0, innerWidth]),
    [allData, xValue, innerWidth]
  );

  const yScale = useMemo(
    () =>
      scaleLog()
        .domain([epsilon, max(allData, yValue)]) // log scale domain cannot start at 0
        .range([innerHeight, 0]),
    [epsilon, allData, yValue, innerHeight]
  );

  const lineGenerator = useMemo(
    () =>
      line()
        .x((d) => xScale(xValue(d)))
        .y((d) => yScale(epsilon + yValue(d))),
    [xScale, yScale, epsilon, xValue, yValue]
  );

  const mostRecentDate = xScale.domain()[1];

  const handleVoronoiHover = useCallback((d) => {
    //console.log(d)
    setActiveCountryName(d.country);
  }, []);

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left}, ${margin.top})`}
      >
        <XAxis
          xScale={xScale}
          innerWidth={innerWidth}
          innerHeight={innerHeight}
        />
        <YAxis
          yScale={yScale}
          innerWidth={innerWidth}
          innerHeight={innerHeight}
        />
        <text
          transform={`translate(${innerWidth / 2},0)`}
          textAnchor="middle"
        >
          Coronavirus Global Deaths Over Time
        </text>
        <VoronoiOverlay
          onHover={handleVoronoiHover}
          innerWidth={innerWidth}
          innerHeight={innerHeight}
          allData={allData}
          lineGenerator={lineGenerator}
        />
        {data.map((countryTimeSeries) => {
          //const rgbColor = `rgb(${Math.random()*255}, ${Math.random()*255}, ${Math.random()*255})`;
          return (
            <path
              className="marker-line"
              //stroke={rgbColor}
              d={lineGenerator(countryTimeSeries)}
            />
          );
        })}
        {activeCountryName ? (
          <path
            className="marker-line active"
            d={lineGenerator(
              data.find(
                (countryTimeSeries) =>
                  countryTimeSeries.countryName ===
                  activeCountryName
              )
            )}
          />
        ) : null}
      </g>
    </svg>
  );
};
